const EventEmitter = require('events');

const StandardStrike = require('../StandardStrike');
const BattleCalc = require('../../BattleCalc');
const { logError } = require('../../logger');

const BurningHands = function(battleState) {
  this.handle = 'burning_hands';
  this.magic = true;
  this.harm = true;

  this.standardStrike = new StandardStrike(this, battleState);

  this.eventEmitter = new EventEmitter();

  this.isLegal = function(actor, victim) {
    return victim.unit;
  };

  this.execute = function(actor, victim, isProc=false) {
    if (!actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay)) {
      return { success: false };
    }

    try {
      const result = this.standardStrike.execute(
        actor,
        victim,
        isProc? 30+actor.level*6 : BattleCalc.getEquippedAbility(actor,'burning_hands').damage,
      );
      result.actorId = actor.uid;
      result.victimId = victim.uid;
      result.isProc = isProc;
      result.abilityHandle = this.handle;
      result.allPieces = battleState.allPieces;

      this.eventEmitter.emit('result', result);
    } catch (err) {
      logError(err, {
        module: 'BurningHands',
        func: 'execute',
        actor,
        victim,
        isProc,
      });

      return { success: false };
    }
  };
};
module.exports = BurningHands;
