const EventEmitter = require('events')

const BattleCalc = require('../../BattleCalc')
const HealEvent = require('../../events/HealEvent')
const { logError } = require('../../logger')

const Suture = function(battleState) {
	this.handle = 'suture'
	this.physical = true
	this.help = true

	this.eventEmitter = new EventEmitter()

	this.isLegal = function( actor, victim ) {
		return victim && victim.unit
	};

	this.execute = function( actor, victim ) {
		if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
			return {success:false};
		}

		try {
			const ability = BattleCalc.getEquippedAbility( actor, this.handle );

			const bleeding = BattleCalc.getCondition( actor, 'bleeding' );
			if( bleeding ) {
				bleeding.potency -= ability.potency;
				if( bleeding.potency <= 0 ) {
					battleState.expireCondition( bleeding.uid );
				} else {
					battleState.updateCondition( victim.uid, bleeding.uid );
				}
			}

			const healEvent = new HealEvent( battleState, actor, ability.healing );
			healEvent.execute( victim );

			var result = {healEvent,bleeding};
			result.abilityHandle = this.handle;
			result.actorId = actor.uid;
			result.victimId = victim.uid;
			result.allPieces = battleState.allPieces;

			this.eventEmitter.emit( 'result', result );
		} catch (err) {
      logError(err, {
        module: 'Suture',
        func: 'execute',
        actor,
        victim
      })

			return { success: false }
		}
	}
}
module.exports = Suture
