const EventEmitter = require('events');

const getUnitState = require('../../../isomorphic-helpers/getUnitState');
const BattleCalc = require('../../BattleCalc');
const Elf = require('../../../models/game_pieces/ai_units/summoned_units/Elf');
const SummonedCondition = require('../../conditions/SummonedCondition');
const { logError } = require('../../logger');

const SummonElf = function(battleState) {
  this.handle = 'summon_elf';
  this.magic = true;
  this.help = true;

  this.eventEmitter = new EventEmitter();

  this.isLegal = function(actor, victim) {
    return victim.tile && !victim.occupied;
  };

  this.execute = function(actor, victim) {
    if (!actor || !actor.inPlay || !victim) {
      return { success: false };
    }

    try {
      var ability = BattleCalc.getEquippedAbility(actor, this.handle);

      const elf = Elf(ability.potency, actor);
      elf.ability_trees = elf.getAbilityTrees();
      elf.attributes = { base: elf.getBaseAttributes() };
      const elf_unit_state = getUnitState({ ai_unit: elf });
      elf_unit_state.x = victim.x;
      elf_unit_state.y = victim.y;

      for (var prop in elf_unit_state.equipment) {
        elf_unit_state.equipment[prop].tint = 0x337433;
      }

      battleState.registerNewUnit(elf_unit_state);
      battleState.onUnitCondition(new SummonedCondition(
        actor,
        elf_unit_state,
        battleState,
        ability.duration,
      ));

      var result = { elf: elf_unit_state };
      result.abilityHandle = this.handle;
      result.actorId = actor.uid;
      result.victimId = victim.uid;
      result.allPieces = battleState.allPieces;
      this.eventEmitter.emit('result', result);
    } catch (err) {
      logError(err, {
        module: 'SummonElf',
        func: 'execute',
        actor,
        victim,
      });
      return { success: false };
    }
  };
};
module.exports = SummonElf;
