const EventEmitter = require('events')

const StandardStrike = require('../StandardStrike')
const BattleCalc = require('../../BattleCalc')
const { logError } = require('../../logger')

const VoidSpike = function( battleState ) {
	this.handle = 'void_spike';
	this.magic = true;
	this.ranged = true;
	this.harm = true;

	this.standardStrike = new StandardStrike( this, battleState );

	this.eventEmitter = new EventEmitter();

	this.isLegal = function( actor, victim ) {
		return true;
	};

	this.execute = function( actor, victim ) {
		if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
			return {success:false};
		}

		setTimeout(() => {
      try {
  			if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
  				return
  			}

  			const result = this.standardStrike.execute( 
  				actor, 
  				victim, 
  				BattleCalc.getEquippedAbility(actor,'void_spike').damage,
  				false
  			);
  			result.abilityHandle = 'void_spike';
  			result.actorId = actor.uid;
  			result.victimId = victim.uid;		
  			result.allPieces = battleState.allPieces;
  			this.eventEmitter.emit( 'result', result );
      } catch (err) {
        logError(err, {
          module: 'VoidSpike',
          func: 'execute',
          actor,
          victim
        })
      }
    }, BattleCalc.getCooldown(actor) * 0.5)
	}
}
module.exports = VoidSpike
