const EventEmitter = require('events');

const getUnitState = require('../../../isomorphic-helpers/getUnitState');
const BattleCalc = require('../../BattleCalc');
const Poppet = require('../../../models/game_pieces/ai_units/summoned_units/Poppet');
const SummonedCondition = require('../../conditions/SummonedCondition');
const { logError } = require('../../logger');

const FauxSelf = function(battleState) {
  this.handle = 'faux_self';
  this.magic = true;
  this.help = true;

  this.eventEmitter = new EventEmitter();

  this.isLegal = function(actor) {
    return twoAdjacentTilesAreUnnocupied();

    function twoAdjacentTilesAreUnnocupied() {
      var numUnoccupied = 0;
      var allAdjacentTiles = battleState.getAdjacentTiles(actor);

      for (var i = 0; i < allAdjacentTiles.length; ++i) {
        if (allAdjacentTiles[i] && !allAdjacentTiles[i].occupied) {
          ++numUnoccupied;
        }
      }

      return numUnoccupied >= 2;
    }
  };

  this.execute = function(actor, victim) {
    if (!actor || !actor.inPlay) {
      return { success: false };
    }

    try {
      var ability = BattleCalc.getEquippedAbility(actor, this.handle);

      // calculate target tiles
      var targetTiles = [];
      var adjacentTiles = battleState.getAdjacentTiles(actor);
      var candidateTiles = [];
      for (var i = 0; i < adjacentTiles.length; ++i) {
        if (!adjacentTiles[i].occupied) {
          candidateTiles.push(adjacentTiles[i]);
        }
      }

      var chance1 = Math.floor(Math.random()*candidateTiles.length);
      targetTiles.push(candidateTiles[chance1]);

      var chance2;
      do { chance2 = Math.floor(Math.random()*candidateTiles.length);
      } while (chance2 === chance1);
      targetTiles.push(candidateTiles[chance2]);

      // make the poppets
      var poppets = [];

      var fu = Poppet(ability.potency, actor);
      fu.ability_trees = fu.getAbilityTrees();
      fu.attributes = { base: fu.getBaseAttributes() };
      let fu_unit_state = getUnitState({ ai_unit: fu });
      // the first one takes the actor's place
      fu_unit_state.x = actor.x;
      fu_unit_state.y = actor.y;
      poppets.push(fu_unit_state);

      fu = Poppet(ability.potency, actor);
      fu.ability_trees = fu.getAbilityTrees();
      fu.attributes = { base: fu.getBaseAttributes() };
      fu_unit_state = getUnitState({ ai_unit: fu });
      fu_unit_state.x = targetTiles[0].x;
      fu_unit_state.y = targetTiles[0].y;
      poppets.push(fu_unit_state);

      // the actor moves to one of the spawn tiles
      actor.x = targetTiles[1].x;
      actor.y = targetTiles[1].y;
      targetTiles[1].occupied = true;

      for (let i = 0; i < poppets.length; ++i) {
        battleState.registerNewUnit(poppets[i]);
        battleState.onUnitCondition(new SummonedCondition(
          actor,
          poppets[i],
          battleState,
          ability.duration,
        ));
      }

      this.eventEmitter.emit('result', {
        success: true,
        abilityHandle: this.handle,
        actorId: actor.uid,
        newActorCoords: { x: actor.x, y: actor.y },
        poppets,
      });

    } catch (err) {
      logError(err, {
        module: 'FauxSelf',
        func: 'execute',
        actor,
        victim,
      });

      return { success: false };
    }
  };
};
module.exports = FauxSelf;
