const EventEmitter = require('events')

const BattleCalc = require('../../BattleCalc')
const BattleConfig = require('../../../constants/BattleConfig')
const DamageEvent = require('../../events/DamageEvent')
const BleedingCondition = require('../../conditions/BleedingCondition')
const { logError } = require('../../logger')

const InfectMind = function( battleState ) {
	this.handle = 'infect_mind';
	this.magic = true;
	this.ranged = true;
	this.harm = true;

	this.eventEmitter = new EventEmitter();

	this.isLegal = function( actor, victim ) {
		return victim && victim.unit;
	};

	this.execute = function( actor, victim ) {
		if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
			return {success:false};
		}
		
		const ability = BattleCalc.getEquippedAbility( actor, this.handle );

		setTimeout(() => {
			try {
        if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
          return
        }

				var bleedDamageDummy = new DamageEvent( battleState, actor, ability.potency, this );
				bleedDamageDummy.applyStanceAdjustments( victim );
						
				// if the unit is already bleeding, simply add to the potency of the extant condition
				var bleeding = BattleCalc.getCondition( victim, 'bleeding' );
				if( bleeding ) {
					bleeding.potency += bleedDamageDummy.damage;
					battleState.updateCondition( victim.uid, bleeding.uid );
				} else {
					bleeding = new BleedingCondition(
						actor,
						victim,
						battleState,
						bleedDamageDummy.damage
					);
					battleState.onUnitCondition( bleeding );
				}		

				var result = {bleeding};
				result.abilityHandle = this.handle;
				result.actorId = actor.uid;
				result.victimId = victim? victim.uid : null;
				result.allPieces = battleState.allPieces;
				this.eventEmitter.emit( 'result', result );
			} catch (err) {
        logError(err, {
          module: 'InfectMind',
          func: 'execute',
          actor,
          victim
        })
			}
			
		}, BattleCalc.projectileTime(BattleConfig.ARROW_SPEED, actor, victim))
	}
}
module.exports = InfectMind
