const EventEmitter = require('events')

const BattleCalc = require('../../BattleCalc')
const CCHandler = require('../../CCHandler')
const LoseMPEvent = require('../../events/LoseMPEvent')
const { logError } = require('../../logger')

const SleightOfHand = function( battleState ) {
	this.handle = 'sleight_of_hand';
	this.magic = true;
	this.ranged = true;
	this.harm = true;

	this.eventEmitter = new EventEmitter();

	this.isLegal = () => true

	this.execute = function( actor, victim ) {
		if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
			return {success:false};
		}
		
		setTimeout( () => {
			try {
        if( !actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay) ) {
          return
        }

				var success = false;
				var loseMP;

				if( !CCHandler.checkForResistMental(victim,battleState) ) {
					var finalPotency = Math.floor( BattleCalc.getEquippedAbility(actor,'sleight_of_hand').potency );
					if( finalPotency > victim.mp ) {
						finalPotency = victim.mp;
					}
					
					loseMP = new LoseMPEvent( battleState, finalPotency );
					loseMP.execute( victim );
					loseMP.actorId = actor.uid;
					loseMP.victimId = victim.uid;

					success = true;
				}

				var result = {success,loseMP};
				result.abilityHandle = this.handle;
				result.actorId = actor.uid;
				result.victimId = victim? victim.uid : null;
				result.allPieces = battleState.allPieces;
				this.eventEmitter.emit( 'result', result );
			} catch (err) {
        logError(err, {
          module: 'sleight_of_hand',
          func: 'execute',
          actor,
          victim
        })
			}
			
		}, BattleCalc.getCooldown(actor) * 0.5)
	}
}
module.exports = SleightOfHand
