const TreeNode = require('../TreeNode');
const Q_most_single_target_damage = require('./Q_most_single_target_damage');
const Q_slash_or_bash = require('./Q_slash_or_bash');
const BattleCalc = require('../../BattleCalc');
const AICalc = require('../AICalc');
const UseAbility = require('../UseAbility');

const Q_state_of_best_target = function(actor, battleState) {
  const node = new TreeNode(actor, battleState);

  // branch indexes
  const LOW_HP_LOW_ARMOR 	= 0;
  const HIGH_HP_LOW_ARMOR	= 1;
  const HIGH_ARMOR		= 2;
  const STUNNED			= 3;

  node.armorBreak = BattleCalc.getEquippedAbility(actor, 'armor_break');
  node.burningHands = BattleCalc.getEquippedAbility(actor, 'burning_hands');
  node.meleeAttack = BattleCalc.getEquippedAbility(actor, 'melee_attack');

  node.branches[ LOW_HP_LOW_ARMOR ] = new Q_most_single_target_damage(actor, battleState);

  if (actor.equipped_abilities[ 'mortal_strike']) {
    node.branches[ STUNNED ] = new UseAbility(actor, 'mortal_strike');
  } else {
    node.branches[ STUNNED ] = node.branches[ LOW_HP_LOW_ARMOR ];
  }

  // we will pass this a reference to our single-target-damage node, in case the necessary abilities are not equipped
  node.branches[ HIGH_HP_LOW_ARMOR ] = new Q_slash_or_bash(actor, battleState, node.branches[LOW_HP_LOW_ARMOR]);

  if (node.armorBreak) {
    node.branches[ HIGH_ARMOR ] = new UseAbility(actor, 'armor_break');
  } else {
    node.branches[ HIGH_ARMOR ] = node.branches[ HIGH_HP_LOW_ARMOR ];
  }

  node.query = function() {
    const calculateTopSingleTargetDamage = () => {
      var burningHandsDamage = this.burningHands? this.burningHands.damage : 0;
      return Math.max(this.meleeAttack.damage, burningHandsDamage);
    };

    const lowHP_lowArmor = () => {
      return 		_bestTargetArmor < 23
					&&	_bestTargetHP < _topSingleTargetDamage*2.5;
    };

    const highHP_lowArmor = () => {
      return		_bestTargetArmor < 23
					&&	_bestTargetHP > _topSingleTargetDamage*2.5;
    };

    var _bestTarget = AICalc.getBestMeleeTarget(this.actor, battleState.getEnemiesOfUnit(this.actor));
    var _bestTargetHP = _bestTarget.hp;
    var _bestTargetArmor = _bestTarget.attributes.armorClass;

    var _topSingleTargetDamage = calculateTopSingleTargetDamage();

    if (BattleCalc.getCondition(_bestTarget,'stunned')) {
      return this.branches[STUNNED].query(_bestTarget);
    } else if (lowHP_lowArmor()) {
      return this.branches[LOW_HP_LOW_ARMOR].query(_bestTarget);
    } else if (highHP_lowArmor()) {
      return this.branches[HIGH_HP_LOW_ARMOR].query(_bestTarget);
    } else if (node.armorBreak && !BattleCalc.getCondition(_bestTarget,'armor_broken')) {
      return this.branches[HIGH_ARMOR].query(_bestTarget);
    } else {
      return this.branches[HIGH_HP_LOW_ARMOR].query(_bestTarget);
    }
  };

  return node;
};
module.exports = Q_state_of_best_target;
