'use strict';

const assert = require('assert');
const {
  getFibonacci,
  MAX_FIB,
} = require('./constants/Balance');
const { logError } = require('./logger');

const getHeroXPGainResults = (roster_hero, xp_gain) => {
  let levels_gained = 0;

  try {
    assert.ok(!isNaN(roster_hero.xp) && roster_hero.xp >= 0, 'invalid hero.xp');
    assert.ok(!isNaN(xp_gain) && xp_gain >= 0, 'invalid xp_gain');

    let dummy_xp = roster_hero.xp + xp_gain;

    for (
      let next_level = roster_hero.level + 1;
      dummy_xp >= getFibonacci(next_level) && next_level <= MAX_FIB;
      ++next_level
    ) {
      if (dummy_xp >= getFibonacci(next_level)) {
        ++levels_gained;
      } else {
        break;
      }
    }
  } catch (err) {
    logError(err, {
      module: 'helpers',
      func: 'getHeroXPGainResults',
      roster_hero,
      xp_gain,
    });
  }

  return {
    levels_gained,
  };
};

const getTeamString = team => {
  let result = '';
  for (const hero of Object.values(team)) {
    result += hero.handle + ':' + hero.level + ',';
  }
  return result;
};

module.exports = {
  getHeroXPGainResults,
  getTeamString,
};
